#!/bin/bash

# this script is intended to be run as "root" user.
if [ "`whoami`" != "root" ] ; then
	echo ">>> Please execute $0 as a root user" ; exit 1
fi

exiting(){
	writeLog "$*"
	exit 1
}

writeLog() {
	echo "[`date +%Y%m%d-%H:%M:%S`] $*"
	echo "[`date +%Y%m%d-%H:%M:%S`] $*" >> $LOGFILE
}

. `dirname $0`/cfg/`basename $0 .sh`.conf || exiting "Error: `dirname $0`/cfg/`basename $0 .sh`.conf not found !"
. `dirname $0`/cfg/checkdependency.conf || exiting "Error: `dirname $0`/cfg/checkdependency.conf not found !"


#Case
case "$1" in 
    install)
		
		
		writeLog ">>> Creating directory tree $HOMEPATH/$DIRNAME"
		su $U -c "mkdir -p $HOMEPATH/$DIRNAME/ascii $HOMEPATH/$DIRNAME/bgp $HOMEPATH/$DIRNAME/config $HOMEPATH/$DIRNAME/data/UPDATES $HOMEPATH/$DIRNAME/log " 
	
		writeLog ">>> Creating/Linking mrt archive"
		mkdir -p $ARCHIVEPATH/$DIRNAME
		chown -R $U:$U $ARCHIVEPATH/$DIRNAME
		rm -rf $HOMEPATH/$DIRNAME/dataArchive
		su $U -c "ln -s $ARCHIVEPATH/$DIRNAME $HOMEPATH/$DIRNAME/dataArchive"
	
		writeLog ">>> Creating mysql index directory ($INDEXDIR/$ORGANIZATION_NAME)"
			mkdir -p $INDEXDIR/$ORGANIZATION_NAME
			chown -R mysql:mysql $INDEXDIR/$ORGANIZATION_NAME
	
		writeLog ">>> Creating mysql data directory ($DATADIR/$ORGANIZATION_NAME)"
			mkdir -p $DATADIR/$ORGANIZATION_NAME
			chown -R mysql:mysql $DATADIR/$ORGANIZATION_NAME
		
		#ESTENSIONE VPLS 12.08.2010 su ROUTER-ID
		writeLog ">>> Creating $HOMEPATH/$DIRNAME/bgp/bgpd.conf"
		su $U -c "cat > $HOMEPATH/$DIRNAME/bgp/bgpd.conf <<- END
		!
		hostname bgpd
		password $PASS
		enable password $PASS
		!
		router bgp $ASN
		bgp router-id $ROUTER_ID
		!
		! IPv4 neighbor 
		END"
	
		for i in ${!IP4[@]}; do 
			echo neighbor ${IP4[i]} remote-as $ASN >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf
			#echo neighbor ${IP4[i]} prefix-list denyOut out >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf 
		done
		
		su $U -c "cat >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf <<- END
		! IPv6 neighbor 
		END"
		
		#ESTENSIONE VPLS 12.08.2010

		su $U -c "cat >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf <<- END
		! L2VPN SIGNALING
		address-family l2vpn
		END"
		for i in ${!IP4[@]}; do 
			echo neighbor ${IP4[i]} activate >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf
		done
		
		su $U -c "cat >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf <<- END
		exit-address-family
		END"
		
		#FINE ESTENSIONE VPLS 12.08.2010
		
		for j in ${!IP6[@]}; do 
			echo neighbor ${IP6[j]} remote-as $ASN >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf
			echo no neighbor ${IP6[j]} activate >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf
			echo address-family ipv6 >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf
			echo neighbor ${IP6[j]} activate >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf
			echo neighbor ${IP6[j]} prefix-list denyOut out >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf 
		done
		

		# ESTENSIONE VPLS 12.08.2010
		#Dump solo di bgp updates e li mettiamo nella cartella updates (la cartella RIB non serve)
		su $U -c "cat >> $HOMEPATH/$DIRNAME/bgp/bgpd.conf <<- END
		!
		ip prefix-list denyOut deny any
		!
		dump bgp updates $HOMEPATH/$DIRNAME/data/UPDATES/updates.%Y%m%d.%H%M $TIMEUPD
		!
		END"
	
	
		writeLog ">>> Creating $HOMEPATH/$DIRNAME/bgp/zebra.conf file"
		su $U -c "cat > $HOMEPATH/$DIRNAME/bgp/zebra.conf <<- END
		!
		hostname Router
		password $PASS
		enable password $PASS
		END"
	
		writeLog ">>> Creating $HOMEPATH/$DIRNAME/config/peering.cfg file"
		su $U -c "cat > $HOMEPATH/$DIRNAME/config/peering.cfg <<- END
		# Port the quagga terminal listens on
		QUAGGA_VTY_PORT=$VTYZEBRAPORT
		# Port bgpd accepts peering connections on
		BGP_PORT=$BGPPORT
		# Port the bgpd terminal listens on
		BGP_VTY_PORT=$VTYBGPPORT
		END"
	
		writeLog ">>> Creating $HOMEPATH/$DIRNAME/config/retriever_ibgp.conf file"
		su $U -c "cat > $HOMEPATH/$DIRNAME/config/retriever_ibgp.conf <<- END
		## Configuration settings for retriever_ibgp.sh ##
		DB_USER=$DBUSER
		DB_NAME=$DBNAME
		DB_PASSWORD=$PASS
		DB_HOST=$DBHOST
		MYSQL_LOAD_DATA=\"$MYSQL_LOAD_DATA\"
		INDEX_DIR=$INDEXDIR/$ORGANIZATION_NAME
		DATA_DIR=$DATADIR/$ORGANIZATION_NAME
		VM_DUMP_UPDATES_PATHS=$HOMEPATH/$DIRNAME/data/UPDATES
		#VM_DUMP_RIB_PATHS=$HOMEPATH/$DIRNAME/data/RIBS
		BGPDUMP_PATH=$HOMEPATH/tools/bgpdump$SYSTEMARCHITECTURE
		RETRIEVER_LOG_FILE=$HOMEPATH/$DIRNAME/log/Retriever.log
		HUMAN_READABLE_FILE_PATH=$HOMEPATH/$DIRNAME/ascii
		ARCHIVES_PATH=$HOMEPATH/$DIRNAME/dataArchive
		LS=ls
		CAT=cat
		PRESERVE_DIR=0
		NO_ARCHIVE=
		
		## Configuration settings for populateMyAsName.sh ##
		#POPULATE_LOG_FILE=$HOMEPATH/$DIRNAME/log/Populate.log
		#HOME_DIR=$HOMEPATH/$DIRNAME
		
		## Configuration settings for cleanUP.sh ##
		HISTORY_DAYS=$HISTORYDAYS
		CLEANUP_LOG_FILE=$HOMEPATH/$DIRNAME/log/cleanUP.log
		
		## Configuration settings for inferNexthopsRouters.sh ##
		#RESULT=$HOMEPATH/$DIRNAME/server/cfg/nhassociation.txt
		#INFER_LOG_FILE=$HOMEPATH/$DIRNAME/log/infer.log
		#TRACEROUTE_PATH=$HOMEPATH/tools/traceroute-nanog$SYSTEMARCHITECTURE
		#ALLY_PATH=$HOMEPATH/tools/ally$SYSTEMARCHITECTURE
		
		END"
	
		if [ ! -f $HOMEPATH/init_routeserver.conf ]; then
			writeLog ">>> Creating $HOMEPATH/init_routeserver.conf file"
			su $U -c "cat > $HOMEPATH/init_routeserver.conf <<- END
			# owner user of collected datas
			U=$U
			# user home
			HOMEPATH=$HOMEPATH
			# location binary files
			BGPD=$BGPD
			QUAGGA=$QUAGGA
			# timezone for the dumps
			export TZ=UTC
			# this is needed to keep the right enviroment
			export HOME=$HOMEPATH
			END"
		fi
			
		writeLog ">>> Installing scripts in $HOMEPATH"
		cp ./scripts/init_routeserver.sh ./scripts/retriever_ibgp.sh ./scripts/cleanUP.sh $HOMEPATH
		
		writeLog ">>> Installing tools in $HOMEPATH/tools"
		if [ ! -d $HOMEPATH/tools ]; then
			mkdir $HOMEPATH/tools
		fi
		
		cp -R ./tools/*$SYSTEMARCHITECTURE $HOMEPATH/tools

	
		#writeLog ">>> Changing permission settings to $HOMEPATH"
		#chown -R $U:$U $HOMEPATH
		
		writeLog ">>> Create and set permission to /var/run/quagga"
		mkdir -p "/var/run/quagga"
		chmod 777 "/var/run/quagga"
		
		writeLog ">>> Creating DB and user permission. I need access to MySql"
		echo -n  "MySql '$MYSQLROOTUSER' user ---> "

    mysql -u $MYSQLROOTUSER -h $DBHOST -p -e "CREATE DATABASE IF NOT EXISTS $DBNAME; "

 	
		writeLog ">>> Installing $DIRNAME service in $U Crontab"
		crontab -u $U -l > crontab.conf
		
		if ! grep -q "MAILTO=" crontab.conf ; then 
			cat >> crontab.conf <<- END
			MAILTO=""
			END
		fi
		
		#Aggiungi a crontab se cambiato a 2 minuti perchè sennò perdiamo i dati dovuto a prob di sincronizzazione
		if ! grep -q "$HOMEPATH/retriever_ibgp.sh $DIRNAME" crontab.conf ; then 
			cat >> crontab.conf <<- END
			#Clean up $DIRNAME DB #
			0 3 * * * $HOMEPATH/cleanUP.sh $DIRNAME
			#Update $DIRNAME DB routing information service#
			*/2 * * * * $HOMEPATH/retriever_ibgp.sh $DIRNAME
			END
		fi

		crontab -u $U crontab.conf
		rm -rf crontab.conf

		writeLog ">>> Installing init_routeserver_"$U"_"$DIRNAME".sh in $SYSTEMSERVICES"
		cat > $SYSTEMSERVICES/init_routeserver_"$U"_"$DIRNAME".sh <<- END
		#!/bin/bash
		
		### BEGIN CHKCONFIG INFO
		#chkconfig: 2345 55 25
		#description: VPN-Collector init routeserver start service
		### END CHKCONFIG INFO
		
		### BEGIN INIT INFO
		# Required-Start: $network mysql
		# Default-Start: 2 3 4 5
		# Default-Stop: 0 1 6
		# Description: VPN-Collector init routeserver start service
		### END INIT INFO
	
		case \$1 in
			start)
				$HOMEPATH/init_routeserver.sh start $DIRNAME
			;;
			stop)
				$HOMEPATH/init_routeserver.sh stop $DIRNAME
			;;
			status)
				$HOMEPATH/init_routeserver.sh status $DIRNAME
			;;
			*)
				echo "Usage: $SYSTEMSERVICES/init_routeserver_${U}_${DIRNAME}.sh {start|stop|status}" >&2
				exit 2
			;;
			esac
		END
		
	chmod +x $SYSTEMSERVICES/init_routeserver_"$U"_"$DIRNAME".sh
		
		case $OS in
		
			debian)
				update-rc.d init_routeserver_"$U"_"$DIRNAME".sh defaults
				;;
			*)
				update-rc.d init_routeserver_"$U"_"$DIRNAME".sh defaults
				update-rc.d iBGPlayServer_"$U"_"$DIRNAME".sh defaults
			;;	
		esac

		writeLog ">>> Done ...."
		rm -f $LOCKFILE
		
		
		;;
	
	
    remove)
		READ="";

		while [ "$READ" != "yes" -a "$READ" != "no" ]
		do
			READ="";			
			echo -n "[`date +%Y%m%d-%H:%M:%S`] >>> I am going to remove iBGPlay ( mrt data, DB data and services ) for organization '"$ORGANIZATION_NAME"' in user '"$U"' under path '"$HOMEPATH"'? Is this correct? ( user and group '"$U"' will never be removed ) [yes / no] ";  			
			read -r READ;
		done 

		if [ "$READ" == "no" ]; then
			exiting ">>> Exiting ..."
		else
			writeLog ">>> Continuing ..."
		fi
		
		writeLog ">>> Stopping init_routeserver service for $U user"
		$SYSTEMSERVICES/init_routeserver_"$U"_"$DIRNAME".sh stop
		
		writeLog ">>> Stopping iBGPlayServer service for $U user"
		$SYSTEMSERVICES/iBGPlayServer_"$U"_"$DIRNAME".sh stop

		writeLog ">>> Removing directory tree $HOMEPATH/$DIRNAME"
		rm -rf $HOMEPATH/$DIRNAME
		
		writeLog ">>> Removing mrt archive"
		rm -rf $ARCHIVEPATH/$DIRNAME
		
		writeLog ">>> Removing DB and user permission"
		echo -n  "MySql '$MYSQLROOTUSER' user ---> "
		mysql -u $MYSQLROOTUSER -h $DBHOST -p -e "DROP DATABASE IF EXISTS $DBNAME; DROP USER $DBUSER;"
		
		writeLog ">>> Removing $DIRNAME service in $U Crontab"
		crontab -u $U -l | grep -v "$DIRNAME" | crontab -u $U -
		
		writeLog ">>> Removing $DIRNAME init_routeserver and iBGPlayServer service for $U user"
		case $OS in
			fedora)
				chkconfig --del init_routeserver_"$U"_"$DIRNAME".sh
				chkconfig --del iBGPlayServer_"$U"_"$DIRNAME".sh
			;;
			debian)
				update-rc.d -f init_routeserver_"$U"_"$DIRNAME".sh remove
				update-rc.d -f iBGPlayServer_"$U"_"$DIRNAME".sh remove
			;;
			suse)
				chkconfig --del init_routeserver_"$U"_"$DIRNAME".sh
				chkconfig --del iBGPlayServer_"$U"_"$DIRNAME".sh
			;;
			gentoo)
				rc-update del iBGPlayServer_gentoo default
				rc-update del init_routeserver_gentoo default
				rm -rf $SYSTEMSERVICES/init_routeserver_gentoo
				rm -rf $SYSTEMSERVICES/iBGPlayServer_gentoo
			;;
			*)
				update-rc.d -f init_routeserver_"$U"_"$DIRNAME".sh remove
				update-rc.d -f iBGPlayServer_"$U"_"$DIRNAME".sh remove
			;;	
		esac		
			
		rm -rf $SYSTEMSERVICES/init_routeserver_"$U"_"$DIRNAME".sh
		rm -rf $SYSTEMSERVICES/iBGPlayServer_"$U"_"$DIRNAME".sh
		
		writeLog ">>> User $U is not removed, if you want to del it run manually 'userdel -r $U'."	
		writeLog ">>> Also remember to unset sudo permission to $U user in /etc/sudoers file by 'visudo'."
		writeLog ">>> Done ...."
		;;

    *)
    	exiting "Usage: $0 {install | remove}"
	;;
esac

