#!/bin/bash

export TZ=UTC

# Handle return values for commands being run in a pipe chain properly
set -o pipefail

if [ $# -lt 1 ]; then
	echo "Usage: $0 working_dir" >&2
	exit 1
fi

INSTANCE="$1"

# Load configuration file and bail if it can't be read
. ~/$INSTANCE/config/retriever_ibgp.conf || exit 1

CURRENTDATE=`date "+%Y%m%d"`
YESTERDAY=`date --date="-1 day" "+%Y%m%d"`
PASTDATE=`date --date="-$HISTORY_DAYS day" "+%Y%m%d"`
SECONDPASTDATE=`date --date="$PASTDATE 00:00:00" "+%s"`

# Exit function
exiting(){
        writeLog "$*"
        exit 1
}

# write log function
writeLog() {
	echo "[`date +%Y%m%d-%H:%M:%S`] $*" >> "$CLEANUP_LOG_FILE"	
	echo "[`date +%Y%m%d-%H:%M:%S`] $*"
}

writeLog "**** Begin cleanUP process ****"

writeLog "Loading update table from $DB_HOST, related to $DB_NAME DB"
UPD=`mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "show tables" | sort | grep upd` || writeLog "ERROR: while loading update table from $DB_HOST, related to $DB_NAME DB"
writeLog "Loading rib table from $DB_HOST, related to $DB_NAME DB"
RIB=`mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "show tables" | sort | grep rib` || writeLog "ERROR: while loading rib table from $DB_HOST, related to $DB_NAME DB"

writeLog "Today is $CURRENTDATE, $HISTORY_DAYS day of data to be store, delete data bedore $PASTDATE"

writeLog "Drop update table (before $PASTDATE) from $DB_HOST, related to $DB_NAME DB"

for i in $UPD; do
	TABLE=`echo $i | awk -F "upd" '{print$2}'`
	if [ $TABLE -lt $PASTDATE ]; then
		writeLog "mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e DROP TABLE IF EXISTS $i"
		mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "DROP TABLE IF EXISTS $i" || writeLog "ERROR: while dropping $i table from $DB_HOST, related to $DB_NAME DB"
	fi	 	
done

writeLog "Drop rib table (before $PASTDATE) from $DB_HOST, related to $DB_NAME DB"

for i in $RIB; do
	TABLE=`echo $i | awk -F "rib" '{print$2}'`
	if [ $TABLE -lt $PASTDATE ]; then
		writeLog "mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e DROP TABLE IF EXISTS $i"
		mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "DROP TABLE IF EXISTS $i" || writeLog "ERROR: while dropping $i table from $DB_HOST, related to $DB_NAME DB"
	fi
done

writeLog "Checking and Optimizing upd${YESTERDAY} table"
CURRENTTABLE=`mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "show tables" | grep upd${YESTERDAY}`
if [ ! -z "$CURRENTTABLE" ]; then
	writeLog "mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME --auto-repair upd${YESTERDAY}"
	mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME --auto-repair upd${YESTERDAY} || writeLog "ERROR: while checking upd${YESTERDAY} table from $DB_HOST, related to $DB_NAME DB"
	writeLog "mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -o upd${YESTERDAY}"
	mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -o upd${YESTERDAY} || writeLog "ERROR: while optimizing upd${YESTERDAY} table from $DB_HOST, related to $DB_NAME DB"
else
	writeLog "Table ${DB_NAME}.upd${YESTERDAY} doesn't exist!"
fi

writeLog "Checking and Optimizing rib${YESTERDAY} table"
CURRENTTABLE=`mysql -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "show tables" | grep rib${YESTERDAY}`
if [ ! -z "$CURRENTTABLE" ]; then
	writeLog "mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME --auto-repair rib${YESTERDAY}"
	mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME --auto-repair rib${YESTERDAY} || writeLog "ERROR: while checking rib${YESTERDAY} table from $DB_HOST, related to $DB_NAME DB"
	writeLog "mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -o rib${YESTERDAY}"
	mysqlcheck -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -o rib${YESTERDAY} || writeLog "ERROR: while optimizing rib${YESTERDAY} table from $DB_HOST, related to $DB_NAME DB"
else
	writeLog "Table ${DB_NAME}.rib${YESTERDAY} doesn't exist!"
fi

writeLog "**** Done ****"

